// DemoC021n1.c
// 
// functions and main
///////////////////////////////////////////////////////////////////////////////


/*-------------------------------------------------------------------------
   Written by: 
   Peter Seng (2005)
   Thorsten Godau (1999) -> ISR, UART routines

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   In other words, you are welcome to use, share and improve this program.
   You are forbidden to forbid anyone else to use, share and improve
   what you give them.   Help stamp out software-hoarding!
-------------------------------------------------------------------------*/


// used MCU
// Atmel At89ls53 or 80c32x2 (256 Byte RAM,  0KByte EPROM, 12KByte Flash)
// 3 Timer, UART, @ 12.0000 MHz


// compiler commandline
// sdcc DemoC021n1.c


// Requirements:
// eMOD-n MCU module
// (eMOD-o RS-232-module and terminal at 9600Baud, 8N1 at serial port)


// Port-map of the MCU
// P0_0 -> unused                                                   
// P0_1 -> unused                                               
// P0_2 -> unused                                             
// P0_3 -> unused
// P0_4 -> unused
// P0_5 -> unused                                             
// P0_6 -> unused                                             
// P0_7 -> unused                                             
// P1_0 -> unused                                                   
// P1_1 -> unused                                                   
// P1_2 -> unused
// P1_3 -> LD1, LED
// P1_4 -> SW1, switch                                                  
// P1_5 -> unused, (MOSI)
// P1_6 -> unused, (MISO)
// P1_7 -> unused, (SCK)
// P2_0 -> unused                                             
// P2_1 -> unused                                             
// P2_2 -> unused                                              
// P2_3 -> unused                                             
// P2_4 -> unused                                             
// P2_5 -> unused                                             
// P2_6 -> unused                                             
// P2_7 -> unused                                             
// P3_0 -> RXD, RS-232 RX-Data                                          
// P3_1 -> TXD, RS-232 TX-Data                                         
// P3_2 -> unused                                            
// P3_3 -> unused                                            
// P3_4 -> unused                                            
// P3_5 -> unused                                            
// P3_6 -> unused
// P3_7 -> unused


// includes
#include "include/8052.h" // Definitions of 8052 registers, SFRs and Bits
#include	"include/8052x2.h" // Definitions of 8052x2 registers, SFRs and Bits
#include "include/DemoC021n1_def.h" // defines
#include "include/DemoC021n1_glob.h" // global
#include "include/DemoC021n1_proto.h" // prototypes


// functions ////////////////////////////////////////////////////////////////////////////


/********************************************************************/
/* Function    : init_MCU()                                     SUB */
/*------------------------------------------------------------------*/
/* Description : Routine initializes the used MCU registers,        */
/*               the timers and the global variables.               */
/*                                                                  */
/*               Baudrate calculation:                              */
/*               Quarz [Hz]                                         */
/*               RCAP2 = 65536 - ( Quarz / ( 32 * Baud ) )          */
/*               Baud = Quarz / (32 * (65536-RCAP2) )               */
/*               11MHz0592 --> RCAP2 == 0xFFDC (9600Bd)             */
/*               12MHz0000 --> RCAP2 == 0xFFD9 (9615Bd)		        */
/*               16MHz0000 --> RCAP2 == 0xFFCC (9615Bd)		        */
/*               24MHz0000 --> RCAP2 == 0xFFB2 (9615Bd)             */
/*                                                                  */
/*               Timer 0/1 autoreload-register THx calculation:     */
/*               Timebase [sec], Quarz [Hz]                         */
/*               THx = 256-(Timebase * Quarz / 12)                  */
/*               Timebase = (256-THx) * 12 / Quarz                  */
/*               11MHz0592 --> THx == 0xA4 (99,826s)               */
/*               11MHz0592 --> THx == 0xD2 (49,913s)               */
/*               12MHz0000 --> THx == 0x9C (100s)                  */
/*               12MHz0000 --> THx == 0xCE (50s)                   */
/*               16MHz0000 --> THx == 0x7B (99,75s)                */
/*               16MHz0000 --> THx == 0xBD (50,25s)                */
/*               24MHz0000 --> THx == 0x38 (100s)                  */
/*               24MHz0000 --> THx == 0x9C (50s)                   */
/*                                                                  */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 10/01 V1.0 Basic routine                           */
/*               10/02 Peter Seng, THx, timebase values             */
/*                     timer O/1 calculation formula corrected      */
/********************************************************************/
void init_MCU(void)
{

	EA   = 0;     // Disable all interrupts
	ET0  = 0;     // Disable Timer 0 interrupt
	ET1  = 0;     // Disable Timer 1 interrupt
	ET2  = 0;     // Disable Timer 2 interrupt
	EX0  = 0;     // Disable External 0 interrupt
	EX1  = 0;     // Disable External 1 interrupt
	ES   = 0;     // Disable Serial interrupt
	TR0  = 0;     // Stop Timer 0
	TR1  = 0;     // Stop Timer 1
	TR2  = 0;     // Stop Baudrate-Generator Timer 2

	// Set all pins to input-mode
	P0   = 0xFF;
	P1   = 0xFF;
	P2   = 0xFF;
	P3   = 0xFF;

	// Timer 0 & 1
	TL0  = 0x00;    // timer0, MUST be initialized for the first time with 0
	TH0  = 0xCE;    // timer0, tick every 50s at 12MHz0000 (unused)
	TL1  = 0x00;    // timer1, MUST be initialized for the first time with 0
	TH1  = 0x9C;    // timer1, tick every 100s at 12MHz0000
	TMOD = 0x22;    // Autoreloadmode for both Timer 0 & 1

	// Timer 2 (used for baudrate-generation)
	C_T2 = 0;       // Timer-Mode
	RCLK = 1;       // |
	TCLK = 1;       // Set mode to Baudrate-Generator

	SM1  = 1;       // |
	SM0  = 0;       // Set UART to 8N1
	REN  = 1;       // Enable UART reception
	RCAP2H = 0xFF;  // 
	RCAP2L = 0xD9;  // Set Baudrate to 9615Bd at fq = 12MHz0000
						 // 19200Bd (19231Bd) -> RCAP2 = 0xFFD9 at fq = 24MHz
						 // 9600Bd (9615Bd) -> RCAP2 = 0xFFB2 at fq = 24MHz
						 // 9600Bd (9600Bd) -> RCAP2 = 0xFFDC at fq = 11MHz0592
	RI  = 0;        // Clear Receive-Interrupt-Flag
	TI  = 0;        // Clear Transmit-Interrupt-Flag

	// Initialize globals
	DELAY    = 0;
	RXB      = 0;
	RXE      = 0;
	TXCNT    = 0;
	TXPOS    = 0;
	BUSY     = 0;

	TR0  = 0;       // Stop Timer 0 (unused)
	TR1  = 1;       // Start Timer 1
	TR2  = 1;       // Start Baudrate-Generator Timer 2
	ET0  = 0;       // Disable Timer 0 interrupt
	ET1  = 1;       // Enable Timer 1 interrupt
	ET2  = 0;       // Disable Timer 2 interrupt
	ES   = 1;       // Enable Serial interrupt
	EA   = 1;       // Enable all interrupts

	return;
}


/********************************************************************/
/* Function    : External0_ISR()                                ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #0 for         */
/*               handling interrupt requests on external pin P3_2.  */
/*               Free for further use.                              */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void External0_ISR(void) interrupt 0
{
	EX0 = 0;

	// Safety-routine
	_asm
	  nop
	_endasm;

	//EX0 = 1;

	return;
}


/********************************************************************/
/* Function    : Timer0_ISR()                                   ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #1 for         */
/*               handling Timer0 interrupts.                        */
/*               Free for further use.                              */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 10/01    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void Timer0_ISR(void) interrupt 1
{

	ET0 = 0;                     // Disable Timer0-interrupt

	// Safety-routine
	_asm
	  nop
	_endasm;

	//ET0 = 1;                     // Enable Timer0-interrupt

	return;
}


/********************************************************************/
/* Function    : External1_ISR()                                ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #2 for         */
/*               handling interrupt requests on external pin P3_3.  */
/*               Free for further use.                              */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void External1_ISR(void) interrupt 2
{

	EX1 = 0;

	// Safety-routine
	_asm
	  nop
	_endasm;

	//EX1 = 1;

	return;
}


/********************************************************************/
/* Function    : Timer1_ISR()                                   ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #3 for         */
/*               handling Timer1 interrupts.                        */
/*               Timer1 ticks every 100s and is used as global     */
/*               delay-timer in delay().                            */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 10/01    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void Timer1_ISR(void) interrupt 3
{

	ET1 = 0;

	if ( DELAY > 0 ) DELAY--;    // Timebase ticks for delay-routine

	ET1 = 1;

	return;
}


/********************************************************************/
/* Function    : Serial_ISR()                                   ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #4 for         */
/*               handling serial interrupts.                        */
/*               Both character rx and tx is interrupt controlled   */
/*               and handled through ring buffers of size BSIZE.    */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void Serial_ISR(void) interrupt 4
{
	ES = 0;

	if (RI)
	{
		RI = 0;                     // Clear Flag
		RXBuffer[RXB] = SBUF;       // Write RX data to buffer
		if ( ++RXB >= BSIZE )       // Check end of buffer
			RXB = 0;                  // Rewind
	}

	if (TI)
	{
		TI = 0;
		BUSY = TXCNT;
		if (BUSY)
		{
			TXCNT--;
			SBUF = TXBuffer[TXPOS++];
			if ( TXPOS >= BSIZE )
				TXPOS = 0;
		}
	}

	ES = 1;

	return;
}


/********************************************************************/
/* Function    : Timer2_ISR()                                   ISR */
/*------------------------------------------------------------------*/
/* Description : Interrupt-Service-Routine on vector #5 for         */
/*               handling Timer2 interrupts.                        */
/*               Routine is not used, because Timer2 generates      */
/*               the baudrate for the serial communication.         */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 10/01    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void Timer2_ISR(void) interrupt 5
{

	ET2 = 0;

	// Safety-routine
	_asm
	  nop
	_endasm;

	//ET2 = 1;

	return;
}


/********************************************************************/
/* Function    : delay(delaytime)                               SUB */
/*------------------------------------------------------------------*/
/* Description : Routine waits (dependening on Timer 1) for about   */
/*               delaytime * 100s. Best for delaytime > 10         */
/*               Attention: tick every 99.826s at 11MHz0592        */
/*                          tick every 100s at 12MHz0000           */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : unsigned int time_100s                            */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*               09/99    V1.1 nop inserted in delay-loop           */
/*               11/99    V1.2 Removed nop for better accuracy      */
/*                                                                  */
/********************************************************************/
void delay(unsigned int time_100us)
{
	if ( time_100us == 0 )
		return; // Return if no delaytime is assigned

	DELAY = time_100us; // Set global delaytime

	while ( DELAY > 0 );

	return;
}


/********************************************************************/
/* Function    : putchar()                                      SUB */
/*------------------------------------------------------------------*/
/* Description : Routine sends the character value via the UART     */
/*               (can also be used by printf)                       */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : unsigned char value    0x00...0xFF                 */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*               09/99    V1.1 nop inserted in wait-loop            */
/*                                                                  */
/********************************************************************/
void putchar(unsigned char value)
{
	while ( TXCNT >= BSIZE )  /* Wait for room in buffer */
	{
		_asm
		 nop
		_endasm;
	}

	ES = 0;

	if ( BUSY )
	{
		TXBuffer[( TXPOS + TXCNT++ ) % BSIZE] = value;
	}
	else
	{
		SBUF = value;
		BUSY = 1;
	}

	ES = 1;

	return;
}


/********************************************************************/
/* Function    : UART_puts(*text)                               SUB */
/*------------------------------------------------------------------*/
/* Description : Prints text to the UART                            */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : unsigned char *text -> pointer to text             */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void UART_puts(unsigned char *text)
{
	while( *text )          // While not end of text
	{
		putchar(*text++);     // Write character and increment position
	}
	return;
}


// print text to UART, optionally add CR and LF
// Parameter:
//		unsigned char *text -> pointer to text
//		unsigned char ucCrLf
//			> 0  ->  add carriage return and line feed (CR LF) ucCrLf times
//			== 0  ->  do not add carriage return and line feed (CR LF)
// Author: Peter Seng (2002)
// Last changes: 2002 september 17
void UART_putsCrLf(unsigned char *text, unsigned char ucCrLf)
{
	unsigned char ucLoop;

	UART_puts(text); // text via UART
	for (ucLoop=0; ucLoop<ucCrLf; ucLoop ++)
	{
		putchar(0x0D); // CR via UART
		putchar(0x0A); // LF via UART
	}
}


// print filtered text to UART
// Parameter:
//		unsigned char *text -> pointer to text
//		unsigned char ucFilter
//			character not to be included in printed text
// Author: Peter Seng (2003)
// Last changes: 2003-05-26
void UART_putsFiltered(unsigned char *text, unsigned char ucFilter)
{
	while( *text ) // while not end of text
	{
		if (*text != ucFilter) // if not filtered out
			putchar(*text++); // write character and increment position
		else
			text++;
	}
	return;
}


/********************************************************************/
/* Function    : RXChar()                                       SUB */
/*------------------------------------------------------------------*/
/* Description : This function returns a character from the         */
/*               receivebuffer if available.                        */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : none                                               */
/*------------------------------------------------------------------*/
/* Returnvalue : unsigned int (range 0x0000..0x0100)                */
/*                                                                  */
/*                     0x0100   (EOF) no character in rxbuffer      */
/*                     0x0000..0x00FF character from UART           */
/*------------------------------------------------------------------*/
/* History     : 06/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
unsigned int RXChar(void)
{
	unsigned char value;        // Declare value variable

	if ( RXE != RXB )           // Character available
	{
		value = RXBuffer[RXE];

		if ( ++RXE >= BSIZE )     // Check end of buffer
			RXE = 0;
	}
	else
	{
		return(NCA);              // No character available, return NCA
	}

	return((unsigned int)value);
}


/********************************************************************/
/* Function    : errormsg(ascii)                                SUB */
/*------------------------------------------------------------------*/
/* Description : This subroutine generates a error message via UART */
/*------------------------------------------------------------------*/
/* Author      : Dipl.-Ing. (FH) Thorsten Godau                     */
/*------------------------------------------------------------------*/
/* Input       : unsigned int ascii -> Character sent via UART      */
/*------------------------------------------------------------------*/
/* Returnvalue : none                                               */
/*------------------------------------------------------------------*/
/* History     : 11/99    V1.0 Basic routine                        */
/*                                                                  */
/********************************************************************/
void errormsg(unsigned char ascii)
{
	putchar(0x0A);
	putchar(0x0D);
	putchar(ascii);
	UART_puts("-error occured !");
	putchar(0x0A);
	putchar(0x0D);
	putchar(0x07);
	return;
}


void MsDelay(unsigned int iMsDelay)
// stop programm execution for iMsDelay * ms
// WARNING: the delay is not very accurate, may last longer, depending on interrupts
// function calibrated to 8052CPU/12MHz0000/12clocksPerCycle
// Parameter:
//		iMsDelay -> delay in milliseconds
// Author: Peter Seng (2002)
// Last changes: 2004 december 17
{
	unsigned int a,b;

	for (a=iMsDelay; a>0; a--)
	{
		for (b=70; b>0; b--) // 12MHz0000 -> b==70, 11MHz0592 -> b==64
		{
			_asm
				nop
			_endasm;
		}
	}
	return;
}


void setTimeOut(unsigned long ul100us)
// set timeout timer, interrupt driven depending on timer 1
// time 100s ... 119h 
// accuracy depending on timer interrupt
// Parameter:
//		ul100us -> timeout time 100s x ul100us
// Author: Peter Seng (2002)
// Last changes: 2002 september 30
{
	if ( ul100us == 0 )
		return; // Return if no delaytime is assigned
	DELAY = ul100us; // Set global delaytime
}


unsigned char checkTimeOut(void)
// check timeout timer, interrupt driven depending on timer 1
// accuracy depending on timer interrupt
// Parameter: none
// Return value:
//		1 -> timeout not expired
//		0 -> timeout expired
// Author: Peter Seng (2002)
// Last changes: 2002 september 30
{
	if ( DELAY > 0 )
		return 1;
	else
		return 0;
}


void LedOn(unsigned char ucState)
// switch Led connected to port of CPU on/off
// Parameter:
//		ucState 
//			==0 -> LED off
//			!=0 -> LED on
// Author: Peter Seng (2002)
// Last changes: 2004 december 17
{
	if (ucState==0)
		Led = 0;
	else
		Led = 1;
	return;
}


void MsgLed(unsigned char ucMsgNr, unsigned char ucLoop)
// show message via flashing LED, amount of flashes identifies message
// Author: Peter Seng (2002)                                         
// Parameter:
//		unsigned char ucMsgNr -> 0x1..0xFF, flash LED ucMsgNr times  
//		unsigned char ucLoop
//			0 -> show message in endless loop (FUNCTION WILL NEVER TERMINATE)
//			0x1..0xFF -> show message ucLoop times   
// Returnvalue : none        
// Last changes: 2002 september 17
{
	unsigned char ucR=0, ucFlash;

	LedOn(0); 
	if (ucLoop == 0)
	{
		ucR=1; // endless loop
		ucLoop++;
	}
	while( ucLoop>0 ) // loop
	{
		MsDelay(1500); // 1500ms pause
		for (ucFlash=1; ucFlash<=ucMsgNr; ucFlash++)
		{
			LedOn(1); // slow flash TEST LED, show message
			MsDelay(444); // 444ms on
			LedOn(0); 
			MsDelay(222); // 222ms off
		}
		if (ucR==1) 
			ucLoop=1; // endless loop
		else
			ucLoop--; // terminated loop
	} // end while
	return;
}


void UART_printIntDot(unsigned char ucDigits, char cShowLz, char cDec, int iA)
// print int value to UART as (1-5 digit + sign + dot) = 3-7 digit decimal value
// show int with specified number of decimal digits as pseudo float value
// Parameter: 
//		ucDigits
//			1..5 number of digits, if < 5 shown value may be incomplete
//		cShowLz 
//			>= 0 suppress leading zeros            ->  ___+5.3     _ == SPACE CHARACTER
//       == 0 show leading zeros                ->  +0005.3
//			<= 0 suppress leading zeros and spaces ->  +5.3
//		cDec -> number of decimal digits
//			0x0 -> sXXXXX.
//			0x1..0x4 -> sXXXX.X, sXXX.XX, sXX.XXX, sX.XXXX
//			0x5 -> sXXXXX  
//    iA -> int to be shown as as pseudo-float value, 7-digit decimal value with sign and dot
// Return value: none
// Author: Peter Seng (2002)
// Last changes: 2003-05-26
{
	unsigned char ucSign, ucSignPrinted=0, ucStoreDigits;
	char cNum;
	unsigned char ucOffset=6;
	unsigned char ucStringInt[8]="       "; // max. 7 character + NULL (EOS) character -> length == 8 characters
		
	ucStringInt[7]=0x0; // init end of string with null
	ucStoreDigits=ucDigits; // make a copy of initial value
	if (iA>=0)
		ucSign=0x2B; // sign +
	else 
		ucSign=0x2D; // sign -
	for (ucDigits;ucDigits>0;ucDigits--)
	{ 
		if (cDec == 0)
		{
			ucStringInt[ucOffset] = 0x2E; // write dot
			ucOffset--;
		}
		cNum = (char)(iA%10); // calculate remainder, type cast
		if (cNum<0)
			cNum=-cNum; // build positive value
		if ( (cShowLz<=0)&&(iA==0)&&(cDec<0) )
		{
			// suppress leading zeros	 AND  digit is leading zero  AND  LeadDigit is printed
			if (ucSignPrinted==0)
			{
				// sign not printed
				ucStringInt[ucOffset] = ucSign;	// write sign and decrement position
				ucOffset--;
				ucSignPrinted=1; // set flag
			}
			ucStringInt[ucOffset] = 0x20; // write blank and decrement position
			ucOffset--;
		}
		else
		{
			// show leading zeros  OR  digit is none leading zero  OR  LeadDigit is not printed
			ucStringInt[ucOffset] = cNum+0x30;	// write character and decrement position
			ucOffset--;
		}
		iA=iA/10; // build next digit
		cDec--; // decrement dot indicator
	}
	if (ucSignPrinted==0)
	{
		// sign not printed
		ucStringInt[ucOffset] = ucSign;	// write sign and decrement position
	}
	if (cShowLz < 0)
	{
		// suppress leading spaces
		UART_putsFiltered(&ucStringInt[0], 0x20); // Message via UART, do not send space character
	}
	else
	{
		// print leading spaces and leading zeros
		UART_puts(&ucStringInt[5-ucStoreDigits]); // Message via UART, only show valid characters
	}
	return;
}





// main /////////////////////////////////////////////////////////////////////////////////
void main( void )
{
	volatile idata unsigned int rxbyte; // contains the byte received from UART
	int iCount=-13;
	unsigned char ucDisable=0;

	init_MCU(); // initialize MCU
	// CKCON = X1; // only for 8052x2 style processors, initialize clock control register, 12-clock mode (default)
	// CKCON = X2; // only for 8052x2 style processors, initialize clock control register, 6-clock mode
	UART_putsCrLf("",5); // Message via UART, 5x(CR+LF)
	UART_putsCrLf(VersionText1,1); // Message via UART, 1x(CR+LF)
	UART_putsCrLf(VersionText2 VersionText3,2); // Message via UART, 2x(CR+LF)
	UART_putsCrLf("UART loop testing enabled",1); // Message via UART, 1x(CR+LF)
	UART_putsCrLf("-> press any key, <ESC> to disable loop testing, <space> bell",1); // Message via UART, 1x(CR+LF)
	while(1) // endless loop, flash LED, I/O via RS-232
	{
		if (0 == checkTimeOut())  // check timeout timer 1, time expired ???
		{
			Led = !Led; // toggle LED P1.3
			if (Switch==1)
				setTimeOut(5000); // set timeout timer 1 to 500ms
			else
				setTimeOut(1000); // set timeout timer 1 to 100ms
			UART_puts("Hello world !  "); // Message via UART
			UART_printIntDot(5,0,5,iCount); // show int value via UART with sign, leading zeros, no decimal digits, no dot
			UART_putsCrLf("",1); // Message via UART, 1x(CR+LF)
			iCount++;
		}
		if (ucDisable==0)
		{
			// UART, loop incoming bytes 
			if ( ( (rxbyte=RXChar()) & 0x100 ) != NCA ) // data received ?
			{
				putchar((unsigned char)(rxbyte & 0x00FF)); // echo incoming bytes
				UART_putsCrLf("",1); // Message via UART, 1x(CR+LF)
			}
			if (rxbyte==0x1B) // ESC pressed
			{
				ucDisable=1; // set Flag
				UART_putsCrLf("UART loop testing disabled",1); // Message via UART, 1x(CR+LF)
			}
			if (rxbyte==0x20) // ESC pressed
				putchar(0x07); // send Ctrl-G (bell) via UART
		}
	} // end while
} // end main
